<?
// $Id: announce.php 97 2005-09-17 01:41:49Z mathias $

// Copyright (c) 2005 Mathias Gug
//
// This program is free software; you can redistribute it and/or 
// modify it under the terms of the GNU General Public License as 
// published by the Free Software Foundation either version 2 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software Foundation, Inc.
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA. 

/**
* Include configuration
*/
require_once 'config.php';

/**
* Include class
*/
require_once 'File/Bittorrent/Encode.php';
require_once 'File/Bittorrent/Decode.php';
require_once 'Log/Log.php';


$Bittorrent_Decode = new File_Bittorrent_Decode;
$Bittorrent_Encode = new File_Bittorrent_Encode;

$log_conf = array('mode' => 0660, 'timeFormat' => '%X %x');
$logger = &Log::singleton('file', '/tmp/announce.php.log', 'announce', $log_conf, $log_level);

$max_return_peers = 20;
$checknat_timeout = 1;

/** Output http response **/
function sendHttpResponse($x) {
	header("Content-Type: text/plain");
	header("Pragma: no-cache");
	logData("bencoded response",$x,PEAR_LOG_DEBUG);
	print($x);
}

/** Fail with an error message **/
function failError($msg) {
	global $Bittorrent_Encode;
	$r = $Bittorrent_Encode->encode(array("failure reason" => $msg));
	sendHttpResponse($r);
	exit();
}

/** Log Data **/
function logData($name,$data,$level) {
  global $logger;
  ob_start();
  var_dump($data);
  $rd = ob_get_contents();
  ob_end_clean();
  $logger->log("$name : $rd",$level);
}


/* get real ip*/
function getIp()
{
	if (getenv("HTTP_CLIENT_IP") && strcasecmp(getenv("HTTP_CLIENT_IP"), "unknown")) 
		$ip = getenv("HTTP_CLIENT_IP");
	else if (getenv("HTTP_X_FORWARDED_FOR") && strcasecmp(getenv("HTTP_X_FORWARDED_FOR"), "unknown"))
		$ip = getenv("HTTP_X_FORWARDED_FOR");
	else if (getenv("REMOTE_ADDR") && strcasecmp(getenv("REMOTE_ADDR"), "unknown"))
		$ip = getenv("REMOTE_ADDR");
	else if (isset($_SERVER['REMOTE_ADDR']) && $_SERVER['REMOTE_ADDR'] && strcasecmp($_SERVER['REMOTE_ADDR'], "unknown"))
		$ip = $_SERVER['REMOTE_ADDR'];
	return($ip);
}
/* Check if peer is nated */
function checkNat() {
  global $ip, $port, $checknat_timeout;

  $nat=0;

  # Try to open peer reported ip
  $fp = fsockopen($ip, $port, $errno, $errstr, $checknat_timeout);
  if (!$fp) {
    # Connection timeout
    $ip = getIp();
    # Try to open auto-detected ip
    $fp = fsockopen(getIp(), $port, $errno, $errstr, $checknat_timeout);
    if (!$fp) {
      # nothing works
      $nat = 1;
    } else {
      fclose($fp);
      $nat = 0;
    }
  } else {
    # the connection works
    fclose($fp);
    $nat = 0;
  }

  return $nat;
}


/** Check URL parameter **/
function checkURL() {
	# Check that parameters are here and set them globally
	$reqparam = "tuid:info_hash:peer_id:!ip:port:uploaded:downloaded:left:!event:!compact:!numwant";
	foreach (explode(":", $reqparam) as $x) {
		$opt = 0; 
		if ($x[0] == "!") {
			$x = substr($x, 1);
			$opt = 1;
		}
	
		if (!isset($_GET[$x])) {
			if (!$opt) { failError("missing key : $x"); }
			continue;
		}

    if (get_magic_quotes_gpc()) 
    {
		  $GLOBALS[$x] = rawurldecode(stripslashes($_GET[$x]));
    } else {
      $GLOBALS[$x] = rawurldecode($_GET[$x]);
    }
	}

	foreach (array("info_hash","peer_id") as $x) {
		if (strlen($GLOBALS[$x]) != 20)
			failError("invalid $x (" . strlen($GLOBALS[$x]) . " - " . $GLOBALS[$x] . ")");
	}

  global $port, $uploaded, $downloaded, $left, $tuid, $event, $ip, $numwant, $max_return_peers;

  if (!is_numeric($port) or $port < 1 or $port > 65536 ) {
    failError("Port wrong : $port");
  }

  if (!is_numeric($uploaded) or !is_numeric($downloaded) or !is_numeric($left)) {
    failError("Wrong stats");
  }

  if (!is_numeric($tuid)) {
    failError("Wrong tuid : $tuid");
  }

  if (isset($numwant)) {
    if (!is_numeric($numwant) or $numwant < 0) {
      failError("Wrong numwant : $numwant");
    } elseif ($numwant < $max_return_peers) {
      $max_return_peers=$numwant;  
    }
  }

  if (isset($event) and $event != "started" and $event != "stopped" and $event != "completed" ) {
    failError("Unsupported event : $event");
  }

  # Set ip if not in the request
  if (!isset($ip)) {
    $GLOBALS['ip'] = getIp();
  }
}

# Fetch the list of peers from mysql to build the peers array
#
# Example of peers array :
#
# array(2) {
#  ["pl4end01fh-qx8gme30F---1"]=>
#    array(3) {
#        ["peer_id"]=>
#	    string(20) "pl4end01fh-qx8gme30F"
#        ["ip"]=>
#	    string(9) "10.0.18.4"
#        ["port"]=>
#	    string(4) "4556"
#      }
#   ["dlo4jw2kfn4lq-idnpou---2"]=>
#     array(3) {
#        ["peer_id"]=>
#            string(20) "dlo4jw2kfn4lq-idnpou"
#        ["ip"]=>
#            string(9) "10.0.18.3"
#        ["port"]=>
#            string(4) "4555"
#      }
# }
function getPeerList() {
  global $announce_interval, $max_return_peers, $dbprfx, $info_hash;

  $ih = mysql_real_escape_string($info_hash);
  $strQuery="SELECT tuid, peer_id , ip, port FROM " . $dbprfx . "torrents_peers WHERE info_hash='$ih' AND current_timestamp() - last_time < 3*$announce_interval AND nat = 0 LIMIT $max_return_peers";
  logData("PEERS SELECT Query",$strQuery,PEAR_LOG_DEBUG);
  $rr = mysql_query($strQuery);
  if (!$rr) {
  	logData("ERROR SELECTING PEERS",mysql_error(),PEAR_LOG_ERR);
  }
  while ( $p = mysql_fetch_assoc($rr)) { 
  	if ($p != FALSE) {
  		$peers[$p["peer_id"] . "---" . $p["tuid"]]=array("peer_id" => $p["peer_id"], "ip" => $p["ip"], "port" => $p["port"]);
  	}
  };
  logData("PEERS LIST",$peers,PEAR_LOG_DEBUG);
  return $peers;
};

function getCompleted($info_hash) {
  $strQuery = "SELECT COUNT(*) FROM " . $dbprfx . "members_torrents_stats WHERE info_hash='$info_hash' AND completed = 1 AND active=1";
  logData("COMPLETED SELECT Query",$strQuery,PEAR_LOG_DEBUG);
  $rr = mysql_query($strQuery);
  if (!$rr) {
  	logData("ERROR GETTING COMPLETED",mysql_error(),PEAR_LOG_ERR);
  }
  $row = mysql_fetch_row($rr);
  return (int) $row[0];
}

function getIncomplete($info_hash) {
  $strQuery = "SELECT COUNT(*) FROM " . $dbprfx . "members_torrents_stats WHERE info_hash='$info_hash' AND completed = 0 AND active=1";
  logData("INCOMPLETE SELECT Query",$strQuery,PEAR_LOG_DEBUG);
  $rr = mysql_query($strQuery);
  if (!$rr) {
  	logData("ERROR GETTING INCOMPLETE",mysql_error(),PEAR_LOG_ERR);
  }
  $row = mysql_fetch_row($rr);
  return (int) $row[0];
}

logData("-------NEW REQUEST----","-----------------",PEAR_LOG_DEBUG);

checkURL();

mysql_connect($dbhost, $dbuser, $dbpass) or failError("Error: connection to mysql failed !");
mysql_select_db($dbname) or failError("Error: connection to database failed !");

$ih = mysql_real_escape_string($info_hash);
$strQuery="SELECT * FROM " . $dbprfx . "torrents WHERE info_hash='$ih'";
logData("TORRENT HASH SELECT Query",$strQuery,PEAR_LOG_DEBUG);
$rr = mysql_query($strQuery);
if (!$rr) {
	logData("ERROR CHECKING TORRENT HASH",mysql_error(),PEAR_LOG_ERR);
}
if (!mysql_fetch_row($rr)) {
	failError("Unknown torrent hash : $info_hash");
}

# Process event
if ("$event" == "started") {
  # register new peer
  $nat = checkNat();
  $ih = mysql_real_escape_string($info_hash);
  $pi = mysql_real_escape_string($peer_id);
  $strQuery="SELECT peer_id FROM " . $dbprfx . "torrents_peers WHERE info_hash='$ih' and peer_id='$pi' and tuid='$tuid'";
  logData("STARTED PEERS SELECT Query",$strQuery,PEAR_LOG_DEBUG);
  $rr = mysql_query($strQuery);
  if (!$rr) {
	  logData("ERROR SELECTING PEERS STARTED",mysql_error(),PEAR_LOG_ERR);
  }
	if ( mysql_num_rows($rr) == 0 ) {
		# Add peer_id to peer list in mysql
		$ih = mysql_real_escape_string($info_hash);
		$pi = mysql_real_escape_string($peer_id);
		$strQuery="INSERT into " . $dbprfx . "torrents_peers(info_hash,peer_id,ip,port,nat,tuid) VALUES ('$ih','$pi','$ip',$port,$nat,$tuid)";
		logData("PEER ID INSERT Query",$strQuery,PEAR_LOG_DEBUG);
		$rr = mysql_query($strQuery);
		if (!$rr) {
			logData("ERROR INSERTING PEERS",mysql_error(),PEAR_LOG_ERR);
		}
    # Check if peer has already statistics
    $strQuery="SELECT tuid FROM " . $dbprfx . "members_torrents_stats WHERE  tuid='$tuid' AND info_hash='$ih'";
    logData("USER STATS SELECT Query",$strQuery,PEAR_LOG_DEBUG);
		$rr = mysql_query($strQuery);
		if (!$rr) {
			logData("ERROR USER STATS SELECT",mysql_error(),PEAR_LOG_ERR);
		}
    if ($left == 0) {
      $completed = 1;
    } else {
      $completed = 0;
    }
    if (mysql_num_rows($rr) != 0) {
  		$strQuery="UPDATE " . $dbprfx . "members_torrents_stats SET completed=$completed, active=1 WHERE tuid='$tuid' AND info_hash='$ih'";
    } else {
  		$strQuery="INSERT INTO " . $dbprfx . "members_torrents_stats(tuid,info_hash,uploaded,downloaded,completed,active) VALUES ($tuid,'$ih',0,0,$completed,1)";
    }
    logData("TORRENT USER STATS INSERT Query",$strQuery,PEAR_LOG_DEBUG);
		$rr = mysql_query($strQuery);
		if (!$rr) {
			logData("ERROR INSERT TORRENT USER STATS",mysql_error(),PEAR_LOG_ERR);
		}
	}
} elseif ("$event" == "completed") {
  # completed event
  $ih = mysql_real_escape_string($info_hash);
	$strQuery="UPDATE " . $dbprfx . "members_torrents_stats SET completed=1 WHERE info_hash='$ih' AND tuid='$tuid'";
	logData("COMPLETED UPDATE Query",$strQuery,PEAR_LOG_DEBUG);
	$rr = mysql_query($strQuery);
	if (!$rr) {
		logData("ERROR UPDATE ON COMPLETED",mysql_error(),PEAR_LOG_ERR);
	}
}

# Update user stats
# need to have the peer in the torrents_peers : after 'started' event, but before 'stopped' event processing
$ih = mysql_real_escape_string($info_hash);
$strQuery="UPDATE " . $dbprfx . "torrents_peers SET uploaded=$uploaded, downloaded=$downloaded, last_time = NULL WHERE info_hash='$ih' AND peer_id='$peer_id' AND tuid='$tuid'";
logData("USER STAT UPDATE Query",$strQuery,PEAR_LOG_DEBUG);
$rr = mysql_query($strQuery);
if (!$rr) {
	logData("ERROR UPDATING USER STATS",mysql_error(),PEAR_LOG_ERR);
}

if ( "$event" == "stopped") {
	# stopped event
  $ih = mysql_real_escape_string($info_hash);
  $strQuery="SELECT peer_id FROM " . $dbprfx . "torrents_peers WHERE info_hash='$ih' AND peer_id='$peer_id' AND tuid='$tuid'";
  logData("STOPPED PEERS SELECT Query",$strQuery,PEAR_LOG_DEBUG);
  $rr = mysql_query($strQuery);
  if (!$rr) {
	  logData("ERROR SELECTING PEERS STOPPED",mysql_error(),PEAR_LOG_ERR);
  }
	if ( mysql_num_rows($rr) != 0 ) {
		# Remove peer_id from the peer list
		# ... in mysql
		$ih = mysql_real_escape_string($info_hash);
		$pi = mysql_real_escape_string($peer_id);
    # update user stats for the torrent
    $strQuery="UPDATE " . $dbprfx . "members_torrents_stats m, " . $dbprfx . "torrents_peers t SET m.active=0 , m.downloaded = m.downloaded + t.downloaded, m.uploaded = m.uploaded + t.uploaded WHERE m.info_hash='$ih' AND m.tuid='$tuid' AND t.info_hash=m.info_hash AND t.peer_id='$pi' AND t.tuid=m.tuid";
		logData("UPDATE STOPPED Query",$strQuery,PEAR_LOG_DEBUG);
		$rr = mysql_query($strQuery);
		if (!$rr) {
			logData("ERROR UPDATE STOPPED",mysql_error(),PEAR_LOG_ERR);
		}
		$strQuery="DELETE FROM " . $dbprfx . "torrents_peers WHERE info_hash='$ih' AND peer_id='$pi' AND tuid='$tuid'";
		logData("PEER DELETE Query",$strQuery,PEAR_LOG_DEBUG);
		$rr = mysql_query($strQuery);
		if (!$rr) {
			logData("ERROR DELETE PEER",mysql_error(),PEAR_LOG_ERR);
		}
	}
}

# Build response
$peers=getPeerList();

# Remove peer id from peer list
if ( array_key_exists($peer_id."---".$tuid,$peers)) {
  unset($peers[$peer_id."---".$tuid]);
}
# Convert all ports string to integers
foreach( array_keys($peers) as $key){
	$p =& $peers[$key];
	$p["port"]=(int) $p["port"];
}
# convert ip to compact
if ($compact == 1) {
  $tmp_peers = '';
  foreach( array_keys($peers) as $key){
    $peer_ip = explode('.', $peers[$key]["ip"]);
		$peer_ip = pack("C*", $peer_ip[0], $peer_ip[1], $peer_ip[2], $peer_ip[3]);
		$peer_port = pack("n*", (int)$peers[$key]["port"]);
    $tmp_peers .= $peer_ip . $peer_port;
  }
  $peers=$tmp_peers;
}
$completed=getCompleted($info_hash);
$incomplete=getIncomplete($info_hash);
if  ($compact == 1) {
  $data=array("peers" => $peers, "interval" => $announce_interval, "complete" => $completed, "incomplete" => $incomplete);
} else {
  $data=array("peers" => array_values($peers), "interval" => $announce_interval, "complete" => $completed, "incomplete" => $incomplete);
}
logData("SENT DATA",$data,PEAR_LOG_DEBUG);

sendHttpResponse($Bittorrent_Encode->encode($data));

?>
