#!/usr/bin/python

# $Id: Peer.py 97 2005-09-17 01:41:49Z mathias $

# Copyright (c) 2005 Mathias Gug
#
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation either version 2 of 
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA. 

import random, string
import SocketServer,socket
import os,signal
import time

class EchoHandler(SocketServer.StreamRequestHandler):
  def handle(self):
    line = self.rfile.readline()
    self.wfile.write(line)
                    

class Peer:

  def __init__(self,peerid=None,ip=None,port=None,tuid=None,startTCPServer=True):
    if not peerid:
      # Generate random peerid
      self.peerid = ''
      chars = string.letters + string.digits
      for i in range(20):
        self.peerid = self.peerid + random.choice(chars)
    else:
      self.peerid = peerid

    if startTCPServer:
        self.ip = '127.0.0.1'
    else:
      if not ip:
        # Generate random ip
        self.ip = ''
        for i in random.sample(range(1,256),4):
          self.ip = self.ip + `i` + '.'
        self.ip=self.ip[:-1]
      else:
        self.ip = ip

    if not port:
      while 1:
        # Generate random port
        self.port = random.choice(range(1024,65000))
        # See if it's already used
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(1.0)
        try:
          s.connect((self.ip,self.port))
        except socket.error:
          break

    else:
      self.port = port

    if not tuid:
      # Generate random tuid
      self.tuid = random.choice(range(1,1000000))
    else:
      self.tuid = tuid

    self.startTCPServer = startTCPServer
    if self.startTCPServer:
      # Fork tcp server
      self.pid = os.fork() 
      if self.pid == 0:
        # client
        s = SocketServer.TCPServer((self.ip,self.port),EchoHandler)
        s.serve_forever()

  def stopTCPServer(self):
    if self.startTCPServer:
      os.kill(self.pid,signal.SIGTERM)
      os.waitpid(self.pid,0)

class PeerSet:

  def __init__(self,nbpeer=10,startTCPServer=False):
    self.peers = []
    for i in range(nbpeer):
      self.peers.append(Peer(startTCPServer=startTCPServer))

  def getPeerSet(self):
    return self.peers

  def stopTCPServer(self):
    for p in self.peers:
      p.stopTCPServer()

