#!/usr/bin/python2

# $Id: PeerTest.py 97 2005-09-17 01:41:49Z mathias $

# Copyright (c) 2005 Mathias Gug
#
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation either version 2 of 
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA. 

import unittest
import Peer
import socket
import time

class PeerTest(unittest.TestCase):

  def setUp(self):
    pass

  def tearDown(self):
    pass

  def testStopTCPServer(self):
    peerid='12345678901234567890'
    p = Peer.Peer(peerid=peerid,startTCPServer=True)
    p.stopTCPServer()
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(1.0)
    self.assertRaises(socket.error,s.connect,(p.ip, p.port))

  def testNotStartTCPServer(self):
    peerid='12345678901234567890'
    p = Peer.Peer(ip='127.0.0.1',peerid=peerid,startTCPServer=False)
    self.assertEqual(p.ip,'127.0.0.1')
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(1.0)
    self.assertRaises(socket.error,s.connect,(p.ip, p.port))
    p.stopTCPServer()

  def testStartTCPServer(self):
    peerid='12345678901234567890'
    p = Peer.Peer(peerid=peerid,startTCPServer=True)
    self.assertEqual(p.ip,'127.0.0.1')
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(1.0)
    try:
      s.connect((p.ip, p.port))
    except socket.error:
      self.fail('connect error')
    p.stopTCPServer()

  def testPeerSetInit(self):
    ps = Peer.PeerSet(3)
    self.assertEquals(len(ps.getPeerSet()),3)
    for p in ps.getPeerSet():
      self.assertEquals(p.startTCPServer,False)
    ps.stopTCPServer()
              
  def testPeerSetStartTCPServer(self):
    ps = Peer.PeerSet(3,startTCPServer=True)
    for p in ps.getPeerSet():
      self.assertEquals(p.startTCPServer,True)
    ps.stopTCPServer()

  def testPeerSetNotStartTCPServer(self):
    ps =  Peer.PeerSet(3,startTCPServer=False)
    for p in ps.getPeerSet():
      self.assertEquals(p.startTCPServer,False)
    self.assertNotEquals(ps.getPeerSet()[0].ip,ps.getPeerSet()[1].ip)
    ps.stopTCPServer()

if __name__ == "__main__":
  unittest.main()
