#!/usr/bin/python2

# $Id: TrackerTest.py 97 2005-09-17 01:41:49Z mathias $

# Copyright (c) 2005 Mathias Gug
#
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation either version 2 of 
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA. 

import unittest
import httplib
from urllib import *
from BitTorrent import bencode, BTFailure
import MySQLdb
import sha
import Peer
import time
import tempfile
import re
import os
import popen2

class TrackerTest(unittest.TestCase):

  checkStats = True

  def setUp(self):
    self.tracker_url = "drupal"
    self.announce_url = "bttracker/announce.php"
    self.user_baseannounce_url = "bttracker/user"
    self.conn = httplib.HTTPConnection(self.tracker_url)
    
    self.seed = { 'peer_id'  : 'pl4end01fh-qx8gme30F',
          'ip' : '127.0.0.1',
          'port' : 4555 ,
          'tuid' : 1 }
    self.leech = { 'peer_id' : 'dlo4jw2kfn4lq-idnpou',
          'ip' : '127.0.0.1',
          'port' : 4556 ,
          'tuid' : 2 }
    self.leech1 = { 'peer_id'  : 'lokdqd01fh-qx8gme30F',
          'ip' : '127.0.0.1',
          'port' : 4557 ,
          'tuid' : 3 }

    self.dbhost = "localhost"
    self.dbuser = "drupaluser"
    self.dbpass = "drupalp"
    self.dbname = "drupal"
    self.dbprfx = ""
    self.db = MySQLdb.connect(host=self.dbhost,user=self.dbuser,passwd=self.dbpass,db=self.dbname)
    self.dbcursor = self.db.cursor()

    self.torrent_file = { 'filename' : 'samba.tar.gz.torrent' }
    metainfo_file = open(self.torrent_file['filename'], 'rb')
    metainfo = bencode.bdecode(metainfo_file.read())
    metainfo_file.close()
    self.torrent_file['info'] = metainfo['info']
    self.torrent_file['info_hash'] = sha.new(bencode.bencode(self.torrent_file['info'])).digest()

    self.dbcursor.execute("DELETE FROM %storrents WHERE info_hash='%s'" % (self.dbprfx,self.torrent_file['info_hash']))
    self.dbcursor.execute("DELETE FROM %storrents_peers WHERE info_hash='%s'" % (self.dbprfx,self.torrent_file['info_hash']))
    self.dbcursor.execute("DELETE FROM %smembers_torrents_stats WHERE info_hash='%s'" % (self.dbprfx,self.torrent_file['info_hash']))

    strQuery = """INSERT INTO %storrents SET info_hash='%s'""" % (self.dbprfx,self.torrent_file['info_hash'])
    self.dbcursor.execute(strQuery)

  def tearDown(self):
    self.conn.close()
    self.dbcursor.execute("DELETE FROM %storrents WHERE info_hash='%s'" % (self.dbprfx,self.torrent_file['info_hash']))
    self.dbcursor.execute("DELETE FROM %storrents_peers WHERE info_hash='%s'" % (self.dbprfx,self.torrent_file['info_hash']))
    self.dbcursor.execute("DELETE FROM %smembers_torrents_stats WHERE info_hash='%s'" % (self.dbprfx,self.torrent_file['info_hash']))
    self.dbcursor.close()
    self.db.close()

  def testFailureReason(self):
    self.conn.request('GET',"/%s" % (self.announce_url))
    r1 = self.conn.getresponse()
    data1 = r1.read()
    self.assert_(bencode.bdecode(data1).has_key('failure reason'))

  def testWrongInputTuid(self):
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,'ttt',quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.conn.request('GET',strRequest)
    r1 = self.conn.getresponse()
    data1 = r1.read()
    self.assert_(bencode.bdecode(data1).has_key('failure reason'))

  def testWrongInputNumwant(self):
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s&numwant=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started','ttt')
    self.conn.request('GET',strRequest)
    r1 = self.conn.getresponse()
    data1 = r1.read()
    self.assert_(bencode.bdecode(data1).has_key('failure reason'))
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s&numwant=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started',-1)
    self.conn.request('GET',strRequest)
    r1 = self.conn.getresponse()
    data1 = r1.read()
    self.assert_(bencode.bdecode(data1).has_key('failure reason'))
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s&numwant=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started',2)
    self.checkTrackerResponse(strRequest)

  def testWrongInputInfoHash(self):
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],'ttt',quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.conn.request('GET',strRequest)
    r1 = self.conn.getresponse()
    data1 = r1.read()
    self.assert_(bencode.bdecode(data1).has_key('failure reason'))

  def testWrongInputPeerId(self):
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),'ttt',quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.conn.request('GET',strRequest)
    r1 = self.conn.getresponse()
    data1 = r1.read()
    self.assert_(bencode.bdecode(data1).has_key('failure reason'))

  def testWrongInputPort(self):
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),'-1',0,0,0,'started')
    self.conn.request('GET',strRequest)
    r1 = self.conn.getresponse()
    data1 = r1.read()
    self.assert_(bencode.bdecode(data1).has_key('failure reason'))
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),'ttt',0,0,0,'started')
    self.conn.request('GET',strRequest)
    r1 = self.conn.getresponse()
    data1 = r1.read()
    self.assert_(bencode.bdecode(data1).has_key('failure reason'))
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),70000,0,0,0,'started')
    self.conn.request('GET',strRequest)
    r1 = self.conn.getresponse()
    data1 = r1.read()
    self.assert_(bencode.bdecode(data1).has_key('failure reason'))

  def testWrongInputStats(self):
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),'dd',0,0,'started')
    self.conn.request('GET',strRequest)
    r1 = self.conn.getresponse()
    data1 = r1.read()
    self.assert_(bencode.bdecode(data1).has_key('failure reason'))
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,'dd',0,'started')
    self.conn.request('GET',strRequest)
    r1 = self.conn.getresponse()
    data1 = r1.read()
    self.assert_(bencode.bdecode(data1).has_key('failure reason'))
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,'dd','started')
    self.conn.request('GET',strRequest)
    r1 = self.conn.getresponse()
    data1 = r1.read()
    self.assert_(bencode.bdecode(data1).has_key('failure reason'))

  def testUnknownTorrent(self):
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],'linfo_hashllllllllll',quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.conn.request('GET',strRequest)
    r1 = self.conn.getresponse()
    data1 = r1.read()
    self.assert_(bencode.bdecode(data1).has_key('failure reason'))

  def testLeechConnect(self):
    peer_seed = Peer.Peer(ip=self.seed['ip'],port=self.seed['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    peer_seed.stopTCPServer()
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,100,'started')
    res = self.checkTrackerResponse(strRequest)
    self.assertEqual(len(res['peers']),1)
    self.assert_(res.has_key('interval'))
    self.assert_(res.has_key('peers'))
    self.assert_(res.has_key('complete'))
    self.assert_(res.has_key('incomplete'))
    self.assertEqual(len(res['peers']),1)
    self.assert_(res['peers'][0].has_key('peer_id'))
    self.assert_(res['peers'][0].has_key('ip'))
    self.assert_(res['peers'][0].has_key('port'))
    self.assertEqual(res['peers'][0]['peer_id'],self.seed['peer_id'])
    self.assertEqual(res['peers'][0]['ip'],self.seed['ip'])
    self.assertEqual(res['peers'][0]['port'],self.seed['port'])
    self.assertEqual(res['complete'],1)
    self.assertEqual(res['incomplete'],1)
    self.assertEquals(self.getNbCompleted(self.torrent_file['info_hash']),1)

  def testRewriteUrl(self):
    announce_url="%s/%s/announce" % (self.user_baseannounce_url,self.seed['tuid'])
    peer_seed = Peer.Peer(ip=self.seed['ip'],port=self.seed['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    peer_seed.stopTCPServer()
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,100,'started')
    res = self.checkTrackerResponse(strRequest)
    self.assert_(res.has_key('interval'))
    self.assert_(res.has_key('peers'))
    self.assert_(res.has_key('complete'))
    self.assert_(res.has_key('incomplete'))
    self.assertEqual(len(res['peers']),1)
    self.assert_(res['peers'][0].has_key('peer_id'))
    self.assert_(res['peers'][0].has_key('ip'))
    self.assert_(res['peers'][0].has_key('port'))
    self.assertEqual(res['peers'][0]['peer_id'],self.seed['peer_id'])
    self.assertEqual(res['peers'][0]['ip'],self.seed['ip'])
    self.assertEqual(res['peers'][0]['port'],self.seed['port'])
    self.assertEqual(res['complete'],1)
    self.assertEqual(res['incomplete'],1)
    self.assertEquals(self.getNbCompleted(self.torrent_file['info_hash']),1)

  def testStartedAction(self):
    peer_seed = Peer.Peer(ip=self.seed['ip'],port=self.seed['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    peer_seed.stopTCPServer()
    peer_leech = Peer.Peer(ip=self.leech['ip'],port=self.leech['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,100,'started')
    self.checkTrackerResponse(strRequest)
    peer_leech.stopTCPServer()
    peer_leech1 = Peer.Peer(ip=self.leech1['ip'],port=self.leech1['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech1['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech1['peer_id']),quote(self.leech1['ip']),quote("%s" % self.leech1['port']),0,0,100,'started')
    res = self.checkTrackerResponse(strRequest)
    peer_leech1.stopTCPServer()
    self.assertEqual(len(res['peers']),2)
    self.assertEqual([res['peers'][0]['peer_id'],res['peers'][1]['peer_id']].sort(),[self.seed['peer_id'],self.leech['peer_id']].sort())
    self.assertEqual([res['peers'][0]['ip'],res['peers'][1]['ip']].sort(),[self.seed['ip'],self.leech['ip']].sort())
    self.assertEqual([res['peers'][0]['port'],res['peers'][1]['port']].sort(),[self.seed['port'],self.leech['port']].sort())
    self.assertEqual(res['complete'],1)
    self.assertEqual(res['incomplete'],2)
    self.assertEquals(self.getNbCompleted(self.torrent_file['info_hash']),1)

  def testSamePeerId(self):
    uploaded=10
    downloaded=20
    size=100
    peer_seed = Peer.Peer(ip=self.seed['ip'],port=self.seed['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    peer_seed.stopTCPServer()
    peer_leech = Peer.Peer(ip=self.leech['ip'],port=self.leech['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,100,'started')
    res = self.checkTrackerResponse(strRequest)
    peer_leech.stopTCPServer()
    self.assertEqual(len(res['peers']),1)
    self.assertEqual(res['peers'][0]['peer_id'],self.seed['peer_id'])
    self.assertEqual(res['peers'][0]['ip'],self.seed['ip'])
    self.assertEqual(res['peers'][0]['port'],self.seed['port'])
    self.assertEqual(res['complete'],1)
    self.assertEqual(res['incomplete'],1)
    self.assertEquals(self.getNbCompleted(self.torrent_file['info_hash']),1)
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),uploaded,downloaded,size-downloaded,'stopped')
    self.checkTrackerResponse(strRequest)
    peer_leech1 = Peer.Peer(ip=self.leech1['ip'],port=self.leech1['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech1['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech1['peer_id']),quote(self.leech1['ip']),quote("%s" % self.leech1['port']),0,0,size,'started')
    res = self.checkTrackerResponse(strRequest)
    peer_leech1.stopTCPServer()
    self.assertEqual(len(res['peers']),1)
    self.assertEqual(res['peers'][0]['peer_id'],self.seed['peer_id'])
    self.assertEqual(res['peers'][0]['ip'],self.seed['ip'])
    self.assertEqual(res['peers'][0]['port'],self.seed['port'])
    self.assertEqual(res['complete'],1)
    self.assertEqual(res['incomplete'],1)
    self.assertEquals(self.getNbCompleted(self.torrent_file['info_hash']),1)

  def testUpdateUserStatsSession(self):
    uploaded=10
    downloaded=20
    size=100
    peer_seed = Peer.Peer(ip=self.seed['ip'],port=self.seed['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    peer_seed.stopTCPServer()
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,size,'started')
    self.checkTrackerResponse(strRequest)
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),uploaded,downloaded,size-downloaded)
    self.checkTrackerResponse(strRequest)
    if self.checkStats:
      strQuery = """SELECT uploaded, downloaded FROM %storrents_peers WHERE info_hash='%s' AND peer_id='%s'""" % (self.dbprfx,self.torrent_file['info_hash'],self.leech['peer_id'])
      self.dbcursor.execute(strQuery)
      r = self.dbcursor.fetchone()
      self.assertEqual(uploaded,r[0])
      self.assertEqual(downloaded,r[1])
    uploaded=30
    downloaded=50
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),uploaded,downloaded,size-downloaded)
    self.checkTrackerResponse(strRequest)
    if self.checkStats:
      strQuery = """SELECT uploaded, downloaded FROM %storrents_peers WHERE info_hash='%s' AND peer_id='%s'""" % (self.dbprfx,self.torrent_file['info_hash'],self.leech['peer_id'])
      self.dbcursor.execute(strQuery)
      r = self.dbcursor.fetchone()
      self.assertEqual(uploaded,r[0])
      self.assertEqual(downloaded,r[1])

  def testUpdateUserStatsNatStopped(self):
    uploaded=10
    downloaded=20
    size=100
    peer_seed = Peer.Peer(ip=self.seed['ip'],port=self.seed['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    peer_seed.stopTCPServer()
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,size,'started')
    self.checkTrackerResponse(strRequest)
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),uploaded,downloaded,size-downloaded,'stopped')
    self.checkTrackerResponse(strRequest)
    if self.checkStats:
      strQuery = """SELECT uploaded, downloaded FROM %smembers_torrents_stats WHERE info_hash='%s' AND tuid='%s'""" % (self.dbprfx,self.torrent_file['info_hash'],self.leech['tuid'])
      self.dbcursor.execute(strQuery)
      r = self.dbcursor.fetchone()
      self.assertEqual(uploaded,r[0])
      self.assertEqual(downloaded,r[1])


  def testUpdateUserStatsOverall(self):
    uploaded=10
    downloaded=20
    size=100
    peer_seed = Peer.Peer(ip=self.seed['ip'],port=self.seed['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    peer_seed.stopTCPServer()
    peer_leech = Peer.Peer(ip=self.leech['ip'],port=self.leech['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,size,'started')
    self.checkTrackerResponse(strRequest)
    peer_leech.stopTCPServer()
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),uploaded,downloaded,size-downloaded,'stopped')
    self.checkTrackerResponse(strRequest)
    uploaded_2=10
    downloaded_2=20
    peer_leech = Peer.Peer(ip=self.leech['ip'],port=self.leech['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,size-downloaded,'started')
    self.checkTrackerResponse(strRequest)
    peer_leech.stopTCPServer()
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),uploaded_2,downloaded_2,size-downloaded-downloaded_2)
    self.checkTrackerResponse(strRequest)
    if self.checkStats:
      strQuery = """SELECT t.uploaded+m.uploaded, t.downloaded+m.downloaded FROM %storrents_peers t,%smembers_torrents_stats m WHERE t.info_hash='%s' AND t.peer_id='%s' AND m.info_hash=t.info_hash AND m.tuid = %s""" % (self.dbprfx,self.dbprfx,self.torrent_file['info_hash'],self.leech['peer_id'],self.leech['tuid'])
      self.dbcursor.execute(strQuery)
      r = self.dbcursor.fetchone()
      self.assertEqual(uploaded+uploaded_2,r[0])
      self.assertEqual(downloaded+downloaded_2,r[1])

  def testStoppedAction(self):
    uploaded=10
    downloaded=20
    size=100
    peer_seed = Peer.Peer(ip=self.seed['ip'],port=self.seed['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    peer_seed.stopTCPServer()
    peer_leech = Peer.Peer(ip=self.leech['ip'],port=self.leech['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,size,'started')
    self.checkTrackerResponse(strRequest)
    peer_leech.stopTCPServer()
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),uploaded,downloaded,size-downloaded,'stopped')
    self.checkTrackerResponse(strRequest)
    peer_leech1 = Peer.Peer(ip=self.leech1['ip'],port=self.leech1['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech1['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech1['peer_id']),quote(self.leech1['ip']),quote("%s" % self.leech1['port']),0,0,size,'started')
    res = self.checkTrackerResponse(strRequest)
    peer_leech1.stopTCPServer()
    self.assertEqual(len(res['peers']),1)
    self.assertEqual(res['peers'][0]['peer_id'],self.seed['peer_id'])
    self.assertEqual(res['peers'][0]['ip'],self.seed['ip'])
    self.assertEqual(res['peers'][0]['port'],self.seed['port'])
    self.assertEqual(res['complete'],1)
    self.assertEqual(res['incomplete'],1)
    self.assertEquals(self.getNbCompleted(self.torrent_file['info_hash']),1)
    if self.checkStats:
      strQuery = """SELECT uploaded, downloaded FROM %smembers_torrents_stats WHERE info_hash='%s' AND tuid='%s'""" % (self.dbprfx,self.torrent_file['info_hash'],self.leech['tuid'])
      self.dbcursor.execute(strQuery)
      r = self.dbcursor.fetchone()
      self.assertEqual(uploaded,r[0])
      self.assertEqual(downloaded,r[1])


  def testRestartDownload(self):
    uploaded=10
    downloaded=20
    size=100
    peer_seed = Peer.Peer(ip=self.seed['ip'],port=self.seed['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    peer_seed.stopTCPServer()
    peer_leech = Peer.Peer(ip=self.leech['ip'],port=self.leech['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,size,'started')
    self.checkTrackerResponse(strRequest)
    peer_leech.stopTCPServer()
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),uploaded,downloaded,size-downloaded,'stopped')
    self.checkTrackerResponse(strRequest)
    peer_leech = Peer.Peer(ip=self.leech['ip'],port=self.leech['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,size-downloaded,'started')
    res = self.checkTrackerResponse(strRequest)
    peer_leech.stopTCPServer()
    self.assertEqual(len(res['peers']),1)
    self.assertEqual(res['peers'][0]['peer_id'],self.seed['peer_id'])
    self.assertEqual(res['peers'][0]['ip'],self.seed['ip'])
    self.assertEqual(res['peers'][0]['port'],self.seed['port'])
    self.assertEqual(res['complete'],1)
    self.assertEqual(res['incomplete'],1)
    self.assertEquals(self.getNbCompleted(self.torrent_file['info_hash']),1)
    if self.checkStats:
      strQuery = """SELECT uploaded, downloaded FROM %smembers_torrents_stats WHERE info_hash='%s' AND tuid='%s'""" % (self.dbprfx,self.torrent_file['info_hash'],self.leech['tuid'])
      self.dbcursor.execute(strQuery)
      r = self.dbcursor.fetchone()
      self.assertEqual(uploaded,r[0])
      self.assertEqual(downloaded,r[1])

  def testCompletedAction(self):
    uploaded=10
    downloaded=20
    size=100
    peer_seed = Peer.Peer(ip=self.seed['ip'],port=self.seed['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    peer_seed.stopTCPServer()
    peer_leech = Peer.Peer(ip=self.leech['ip'],port=self.leech['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,size,'started')
    self.checkTrackerResponse(strRequest)
    peer_leech.stopTCPServer()
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),uploaded,downloaded,size-downloaded)
    self.checkTrackerResponse(strRequest)
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),uploaded,size,0,'completed')
    self.checkTrackerResponse(strRequest)
    peer_leech1 = Peer.Peer(ip=self.leech1['ip'],port=self.leech1['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech1['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech1['peer_id']),quote(self.leech1['ip']),quote("%s" % self.leech1['port']),0,0,100,'started')
    res = self.checkTrackerResponse(strRequest)
    peer_leech1.stopTCPServer()
    self.assertEqual(len(res['peers']),2)
    self.assertEqual([res['peers'][0]['peer_id'],res['peers'][1]['peer_id']].sort(),[self.seed['peer_id'],self.leech['peer_id']].sort())
    self.assertEqual([res['peers'][0]['ip'],res['peers'][1]['ip']].sort(),[self.seed['ip'],self.leech['ip']].sort())
    self.assertEqual([res['peers'][0]['port'],res['peers'][1]['port']].sort(),[self.seed['port'],self.leech['port']].sort())
    self.assertEqual(res['complete'],2)
    self.assertEqual(res['incomplete'],1)
    self.assertEquals(self.getNbCompleted(self.torrent_file['info_hash']),2)
    if self.checkStats:
      strQuery = """SELECT uploaded, downloaded FROM %storrents_peers WHERE info_hash='%s' AND peer_id='%s'""" % (self.dbprfx,self.torrent_file['info_hash'],self.leech['peer_id'])
      self.dbcursor.execute(strQuery)
      r = self.dbcursor.fetchone()
      self.assertEqual(uploaded,r[0])
      self.assertEqual(size,r[1])

  def testMaxPeerLimitTracker(self):
    trackermaxpeer = 20
    # get peer set
    ps = Peer.PeerSet(trackermaxpeer+2,startTCPServer=True)
    for p in ps.getPeerSet():
      strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,p.tuid,quote(self.torrent_file['info_hash']),quote(p.peerid),p.ip,p.port,0,0,0,'started')
      self.checkTrackerResponse(strRequest)
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,100,'started')
    res = self.checkTrackerResponse(strRequest)
    self.assertEqual(len(res['peers']),trackermaxpeer)
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s&numwant=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,100,'started',trackermaxpeer+2)
    res = self.checkTrackerResponse(strRequest)
    self.assertEqual(len(res['peers']),trackermaxpeer)
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s&numwant=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,100,'started',0)
    res = self.checkTrackerResponse(strRequest)
    self.assertEqual(len(res['peers']),0)
    ps.stopTCPServer()

  def testMaxPeerLimitPeer(self):
    peermaxpeer = 4
    # get peer set
    ps = Peer.PeerSet(peermaxpeer+2,startTCPServer=True)
    for p in ps.getPeerSet():
      strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,p.tuid,quote(self.torrent_file['info_hash']),quote(p.peerid),p.ip,p.port,0,0,0,'started')
      self.checkTrackerResponse(strRequest)
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s&numwant=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,100,'started',peermaxpeer)
    res = self.checkTrackerResponse(strRequest)
    self.assertEqual(len(res['peers']),peermaxpeer)
    ps.stopTCPServer()

  def testCompactExtension(self):
    peer_seed = Peer.Peer(ip=self.seed['ip'],port=self.seed['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    peer_seed.stopTCPServer()
    peer_leech = Peer.Peer(ip=self.leech['ip'],port=self.leech['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s&compact=1" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,100,'started')
    self.checkTrackerResponse(strRequest)
    peer_leech.stopTCPServer()
    peer_leech1 = Peer.Peer(ip=self.leech1['ip'],port=self.leech1['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s&compact=1" % (self.announce_url,self.leech1['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech1['peer_id']),quote(self.leech1['ip']),quote("%s" % self.leech1['port']),0,0,100,'started')
    res = self.checkTrackerResponse(strRequest)
    peer_leech1.stopTCPServer()
    self.assertEqual([unquote(res['peers'][0:6]),unquote(res['peers'][6:12])].sort(),[self.compact_peer_info(self.leech['ip'],self.leech['port']),self.compact_peer_info(self.seed['ip'],self.seed['port'])].sort())

  def testTimeStampUpdate(self):
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    strQuery = """SELECT UNIX_TIMESTAMP(t.last_time), UNIX_TIMESTAMP(m.last_time) FROM %storrents_peers t, %smembers_torrents_stats m WHERE m.info_hash='%s' AND m.tuid='%s' AND t.info_hash='%s' AND t.peer_id = '%s'""" % (self.dbprfx,self.dbprfx,self.torrent_file['info_hash'],self.seed['tuid'],self.torrent_file['info_hash'],self.seed['peer_id'])
    self.dbcursor.execute(strQuery)
    r = self.dbcursor.fetchone()
    self.assert_(len(r) == 2)
    t_last_time = r[0]
    m_last_time = r[1]
    self.assert_(abs(t_last_time - int(time.time())) <= 1)
    self.assert_(abs(m_last_time - int(time.time())) <= 1)
    time.sleep(2)
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0)
    self.checkTrackerResponse(strRequest)
    strQuery = """SELECT UNIX_TIMESTAMP(t.last_time), UNIX_TIMESTAMP(m.last_time) FROM %storrents_peers t, %smembers_torrents_stats m WHERE m.info_hash='%s' AND m.tuid='%s' AND t.info_hash='%s' AND t.peer_id = '%s'""" % (self.dbprfx,self.dbprfx,self.torrent_file['info_hash'],self.seed['tuid'],self.torrent_file['info_hash'],self.seed['peer_id'])
    self.dbcursor.execute(strQuery)
    r = self.dbcursor.fetchone()
    self.assert_(t_last_time < r[0])
    self.assert_(m_last_time == r[1])

  def testNatCheck(self):
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    strQuery = """SELECT nat FROM %storrents_peers WHERE info_hash='%s' AND peer_id = '%s'""" % (self.dbprfx,self.torrent_file['info_hash'],self.seed['peer_id'])
    self.dbcursor.execute(strQuery)
    r = self.dbcursor.fetchone()
    self.assertEquals(r[0],1)
    peer_leech = Peer.Peer(ip=self.leech['ip'],port=self.leech['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,100,'started')
    res = self.checkTrackerResponse(strRequest)
    peer_leech.stopTCPServer()
    peer_leech1 = Peer.Peer(ip=self.leech1['ip'],port=self.leech1['port'])
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech1['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech1['peer_id']),quote(self.leech1['ip']),quote("%s" % self.leech1['port']),0,0,100,'started')
    res = self.checkTrackerResponse(strRequest)
    peer_leech1.stopTCPServer()
    strQuery = """SELECT nat FROM %storrents_peers WHERE info_hash='%s' AND peer_id = '%s'""" % (self.dbprfx,self.torrent_file['info_hash'],self.leech['peer_id'])
    self.dbcursor.execute(strQuery)
    r = self.dbcursor.fetchone()
    self.assertEquals(r[0],0)
    self.assertEquals(len(res['peers']),1)
    self.assertEquals(res['peers'][0]['peer_id'],self.leech['peer_id'])

  def testExpungePeers(self):
    announce_interval = 1
    (tmpcfgf, tmpcfgfilename) = tempfile.mkstemp('.php','config')
    (tmpexpf, tmpexpfilename) = tempfile.mkstemp('.php','expungePeers')
    annoucere = re.compile('^\$announce_interval.+$')
    for l in open('../config.php'):
      l = annoucere.sub("$announce_interval= %s;" % announce_interval,l)
      os.write(tmpcfgf,l)
    os.close(tmpcfgf)

    includecfgre = re.compile("^require_once\s'config.php';$")
    for l in open('../expungePeers.php'):
     l = includecfgre.sub("require_once '%s';" % tmpcfgfilename,l)
     os.write(tmpexpf,l)
    os.close(tmpexpf)

    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.seed['tuid'],quote(self.torrent_file['info_hash']),quote(self.seed['peer_id']),quote(self.seed['ip']),quote("%s" % self.seed['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)
    time.sleep(3*announce_interval)
    strRequest = "/%s?tuid=%s&info_hash=%s&peer_id=%s&ip=%s&port=%s&uploaded=%s&downloaded=%s&left=%s&event=%s" % (self.announce_url,self.leech['tuid'],quote(self.torrent_file['info_hash']),quote(self.leech['peer_id']),quote(self.leech['ip']),quote("%s" % self.leech['port']),0,0,0,'started')
    self.checkTrackerResponse(strRequest)

    os.system("php -d include_path=%s:/usr/lib/php %s" % (os.path.dirname(os.getcwd()),tmpexpfilename))

    os.unlink(tmpcfgfilename)
    os.unlink(tmpexpfilename)

    strQuery = """SELECT peer_id FROM %storrents_peers""" % (self.dbprfx)
    self.dbcursor.execute(strQuery)
    r = self.dbcursor.fetchall()
    self.assertEquals(len(r),1)
    self.assertEquals(r[0][0],self.leech['peer_id'])

  def checkTrackerResponse(self,strRequest):
    self.conn.request('GET',strRequest)
    response = self.conn.getresponse().read()
    try:
      res=bencode.bdecode(response)
    except BTFailure, btMsg:
      self.fail("BTFailure : %s : %s" % (btMsg,response))
    if res.has_key('failure reason'):
      self.fail("%s : %s" % (strRequest,res['failure reason']))
    return res

  def getNbCompleted(self,info_hash):
    strQuery = """SELECT COUNT(*) FROM %smembers_torrents_stats WHERE info_hash='%s' AND completed = 1""" % (self.dbprfx,self.torrent_file['info_hash'])
    self.dbcursor.execute(strQuery)
    r = self.dbcursor.fetchone()
    if r == None:
      return 0
    else:
      return r[0]

  def compact_peer_info(self,ip, port):
    try:
        s = ( ''.join([chr(int(i)) for i in ip.split('.')])
              + chr((port & 0xFF00) >> 8) + chr(port & 0xFF) )
        if len(s) != 6:
            s = ''
    except:
        s = ''  # not a valid IP, must be a domain name
    return s

def suite():
  suite = unittest.TestSuite()
  suite.addTest(TrackerTest("testExpungePeers"))
  #suite.addTest(TrackerTest("testMaxPeerLimitTracker"))
  return suite

if __name__ == "__main__":
  #unittest.TextTestRunner(verbosity=2).run(suite())
  unittest.main()
